@@ -1192,7 +1192,7 @@ reset_hung_process (DBusPendingCall *pending, void *data)
     if (!strcmp (l->data, dest))
     {
       g_free (l->data);
-      hung_processes = g_slist_remove (hung_processes, l->data);
+      hung_processes = g_slist_remove (hung_processes, data);
       break;
     }
   }
@@ -1218,7 +1218,7 @@ reset_hung_process_from_ping (DBusPendingCall *pending, void *data)
     if (!strcmp (l->data, data))
     {
       g_free (l->data);
-      hung_processes = g_slist_remove (hung_processes, l->data);
+      hung_processes = g_slist_remove (hung_processes, data);
       break;
     }
   }
